/*******************************************************************************
*
*                 USING TIMER 0 IN NORMAL MODE WITH OVERFLOW INTERRUPT
*                 ----------------------------------------------------
*
*   Filename:                                    
*   Autor:                  David Kress
*
*   Version:                1.0
*   Date:                   10.11.2014
*   Built with:             ATMEL AVR Studio 4.14 and AVR-GCC; Optimization -01
*
*   Description:            This programm is a simple example how to operate
*                           Timer 0 in normal mode with overflow interrupt.
*                           Every time the timer overflows, the interrupt
*                           subroutine is executed.Every time the subroutine
*                           is startet, PA0 is toggeled.By counting the overflows,
*                           a slow toggeling signal is generated on PA1.
*                           
*                           
*   What you can learn:     - Start Timer 0 in normal mode
*                           - Use the overflow interrupt
*
*   Clock source:           - Any 8Mhz clock
*
*   Recommended             - High: 0x99
*   Fuse settings:          - Low:  0xE4  
*
*
*
*   Schematic:       ___________________
*                   |      ATMEGA32     |
*          Vcc ---->|Vcc                |
*          Vcc ---->|RESET              |
*                   |                   |
*          GND ---->|GND                |
*                   |                   |
*                   |                PA0|----> DOUT
*                   |                PA1|----> DOUT
*                   |                   |
*
*******************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>


volatile unsigned int overflowCounter = 0;  // Conuts the overflows

int main(void)
{
    DDRA = 0x03;                            // Setup PA0 and PA1 as output
    
    TIMSK |= (1<<TOIE0);                    // Enable Timer 0 Overflow interrupt
    TCCR0 |= (1<<CS00)|(1<<CS02);           // Start timer0 with prescaler 1024
    sei();                                  // Set the I-bit in SREG

    for(;;);                                // Endless loop;
                                            // main() will never be left

    return 0;                               // This line will never be executed

}

// Interrupt subroutine for timer 0 overflow
ISR(TIMER0_OVF_vect)							
{
	overflowCounter++;                      // Increment overflowcounter	

    PORTA ^= 0x01;                          // Toggle PA0

	switch (overflowCounter)                // Switch overflowcounter				
	{
        case 10:                            // Is the overflowcounter = 10?
		    PORTA ^= 0x02;                  // Toggle PA1
            overflowCounter = 0;            // Reset overflowcounter
            break;                          // Leave the switch block

        default:                            // If the overflowcounter is not 10
		    break;                          // Just leave the switch block
	}		

}                                           // End of the tomer o overflow
                                            // interrupt subroutine
